﻿namespace SoapEndpoint.Tester.Infrastructure;

public static class Environments
{
    public static Environment DefaultEnvironment { get; private set; }

    public static IList<Environment> Items { get; } = new List<Environment>();

    public static Environment GetByName(string name)
    {
        return Items.FirstOrDefault(e => e.Name == name);
    }

    public static void InitializeEnvironments()
    {
        var environmentNames = Program.Configuration["EnvironmentNames"];
        var environmentUrls = Program.Configuration["EnvironmentUrls"];

        Initialize(environmentNames, environmentUrls);

        var defaultEnvironmentName = Program.Configuration["DefaultEnvironment"];
        DefaultEnvironment = GetByName(defaultEnvironmentName);
    }

    private static void Initialize(string names, string urls)
    {
        var envNames = names.Split(',');
        var envUrls = urls.Split(',');

        if (envNames.Length != envUrls.Length)
        {
            throw new InvalidOperationException();
        }

        for (var i = 0; i < envNames.Length; i++)
        {
            var env = new Environment(envNames[i].Trim(), envUrls[i].Trim());
            Items.Add(env);
        }
    }
}