﻿using SoapEndpoint.Service.Systeem.v1.LogbookService;
using Infrastructure.Utils.Xml;
using SoapEndpoint.Service.Flows.Schuldbemiddeling.v1.SeService;
using SoapEndpoint.Service.Flows.Schuldbemiddeling.v1.ShvService;
using SoapEndpoint.Service.Proxy.Infrastructure;
using SoapEndpoint.Service.Proxy.Models.Flows;
using SoapEndpoint.Tester.Infrastructure;
using BerichtHeaderSE = SoapEndpoint.Service.Flows.Schuldbemiddeling.v1.SeService.BerichtHeaderSE;
using BerichtHeaderSHV = SoapEndpoint.Service.Flows.Schuldbemiddeling.v1.ShvService.BerichtHeaderSHV;
using Saldoverzoek = SoapEndpoint.Service.Flows.Schuldbemiddeling.v1.ShvService.Saldoverzoek;
using SaldoverzoekAntwoord = SoapEndpoint.Service.Flows.Schuldbemiddeling.v1.SeService.SaldoverzoekAntwoord;

namespace SoapEndpoint.Tester.Scenario.Implementations.Schuldbemiddeling;

public class TypedProxyTest : SchuldbemiddelingScenarioBase
{
    public override string Name => "Typed proxy scenario";

    protected override async Task ExecuteAsync()
    {
        StartNewScenario("Typed proxy scenario SHV en SE service v1");

        if (Manager.SystemVersion.Major != SystemVersions.V1_0.Major)
        {
            Manager.LogAction("Skipping scenario. Scenario only supports v1.");
            return;
        }

        var request = await SendSaldoverzoekAsync();
        await SendSaldoverzoekAntwoordAsync(request);
    }

    private async Task<SendSaldoverzoekRequest> SendSaldoverzoekAsync()
    {
        var url = Manager.UrlInfo.GetSystemUrl(ServiceUrlInfo.ShvService, Manager.Environment,
            Manager.SystemVersion);

        var client =
            ServiceClientHelper.ConfigureClient(new SoapShvServiceClient(), Manager.ShvCertificate, url);

        var saldoVerzoekItem = Manager.ResourceManager.GetSingleResource(FlowType.Schuldbemiddeling,
            Manager.FlowVersion, Berichttype.SHV_Saldoverzoek);
        var saldoVerzoek = DataContractSerializeHelper.Deserialize<Saldoverzoek>(
            saldoVerzoekItem.Content);

        var request = new SendSaldoverzoekRequest(GetShvHeader(), saldoVerzoek);

        var response = await client.SendSaldoverzoekAsync(request);

        if (!response.SendSaldoverzoekResult.Success)
        {
            throw new InvalidOperationException();
        }

        return request;
    }

    private async Task SendSaldoverzoekAntwoordAsync(SendSaldoverzoekRequest shvRequest)
    {
        var url = Manager.UrlInfo.GetSystemUrl(ServiceUrlInfo.SeService, Manager.Environment,
            Manager.SystemVersion);

        var client =
            ServiceClientHelper.ConfigureClient(new SoapSeServiceClient(), Manager.SeCertificate, url);

        var saldoVerzoekAntwoordItem = Manager.ResourceManager.GetSingleResource(FlowType.Schuldbemiddeling,
            Manager.FlowVersion, Berichttype.SE_SaldoverzoekAntwoord);
        var saldoVerzoekAntwoord = DataContractSerializeHelper.Deserialize<SaldoverzoekAntwoord>(
            saldoVerzoekAntwoordItem.Content);

        var request = new SendSaldoverzoekAntwoordRequest(GetSeHeader(shvRequest), saldoVerzoekAntwoord);

        var response = await client.SendSaldoverzoekAntwoordAsync(request);

        if (!response.SendSaldoverzoekAntwoordResult.Success)
        {
            throw new InvalidOperationException();
        }
    }

    private BerichtHeaderSHV GetShvHeader() =>
        new()
        {
            Versie = Manager.SystemVersion.ToString(),
            Ontvanger = Manager.Se.Referentienummer,
            Referentienummer = Guid.NewGuid(),
            ConversatieReferentienummer = Guid.NewGuid()
        };

    private BerichtHeaderSE GetSeHeader(SendSaldoverzoekRequest request) =>
        new()
        {
            Versie = Manager.SystemVersion.ToString(),
            Referentienummer = Guid.NewGuid(),
            CrossReferentienummer = request.Header.Referentienummer
        };
}