﻿using System.Resources;
using SoapEndpoint.Service.Proxy.Models.Flows;

namespace SoapEndpoint.Service.Proxy.Infrastructure;

public class ResourceManagerModel : IEquatable<ResourceManagerModel>
{
    public ResourceManagerModel(FlowType flowType, Version version)
    {
        FlowType = flowType;
        Version = version;

        ResourceManager = new ResourceManager(
            $"SoapEndpoint.Service.Proxy.Resources.{Enum.GetName(typeof(FlowType), flowType)}.v{Version.Major}.Messages_v{VersionAsString.Replace('.', '_')}",
            typeof(ResourceManagerModel).Assembly);
    }

    public string VersionAsString => Version.ToString();
    public FlowType FlowType { get; }
    public Version Version { get; }
    public ResourceManager ResourceManager { get; }

    public bool Equals(ResourceManagerModel? other) => VersionAsString == other?.VersionAsString;

    public override string ToString() => VersionAsString;

    public static bool operator ==(ResourceManagerModel a, ResourceManagerModel b) =>
        a.VersionAsString == b.VersionAsString;

    public static bool operator !=(ResourceManagerModel a, ResourceManagerModel b) => !(a == b);

    public override bool Equals(object? obj) => Equals(obj as ResourceManagerModel);

    public override int GetHashCode() => VersionAsString.GetHashCode();
}